import pygame, time, random, json

from gameClasses import *

pygame.display.init()
pygame.font.init()
screen = pygame.display.set_mode((600, 600))
Clock = pygame.time.Clock()
moneyfont = pygame.font.SysFont("Helvetica", 12, True)
shopfont = pygame.font.SysFont("Verdana", 14, True)
countfont = pygame.font.SysFont("Verdana", 8, True)
endfont = pygame.font.SysFont("Helvetica", 100, True)
againfont = pygame.font.SysFont("Helvetica", 40, True)

running = True
framenum = 0

COND_NORMAL = 0
COND_WIN = 1
COND_LOSE = 2

currentcondition = 0

def loadForces():
    with open("gameobjects/forces.json",'r') as f:
        jsobj = json.loads(f.read())
    
    return jsobj

def loadMoneyMakers():
    mm = []
    with open("gameobjects/moneymakers.json",'r') as f:
        jsobj = json.loads(f.read())
    
    for i in jsobj:
        mm.append(Buyable(**i))
    
    return mm

def loadPlanetMods():
    pm = []
    with open("gameobjects/planetmods.json",'r') as f:
        jsobj = json.loads(f.read())
    
    for i in jsobj:
        if 'priceinc' not in i:
            i['priceinc'] = 10.0
        pm.append(Buyable(**i))
    
    return pm

def addForce(f):
    global forces, moon
    f = forces[f]
    f = Force(**f)
    moon.forces.append(f)
    return f

def moneyClick():
    global money, moneylastclick, framenum, moon
    if moneylastclick+6 < framenum: #Lets limit money income some
        money += 1+int(getIncome()*0.07)
        moneylastclick = framenum
        
        addForce('Click')

def buyShop(index):
    global money, moneymakers, properties
    mm = moneymakers[index]
    if money >= mm.currentprice:
        money -= mm.currentprice
        properties[index] += 1
        mm.adjustprice(properties[index])
        if mm.force != "":
            addForce(mm.force)

def buyMod(index):
    global money, planetmods, activemods
    pm = planetmods[index]
    if money >= pm.currentprice:
        money -= pm.currentprice
        activemods[index].append(addForce(pm.force))
        pm.adjustprice(len(activemods[index]))
        

def getIncome():
    global moon,moneymakers,properties,planetmods,activemods
    income = 0
    for i,mm in enumerate(moneymakers):
        income += mm.income*properties[i]
    for i,pm in enumerate(planetmods):
        income += pm.income*len(activemods[i])
    
    income -= income*int((moon.red+moon.blue)/300.0)
    return income
    
moon = Moon()
forces = loadForces()
simpleforces = ['RedShift','BlueShift','NiceRedShift','NiceBlueShift']

moneymakers = loadMoneyMakers()
properties = [0]*len(moneymakers)

planetmods = loadPlanetMods()
activemods = [[] for i in xrange(len(planetmods))]

money = 20
moneylastclick = 0

def reset():
    moon = Moon()
    forces = loadForces()
    simpleforces = ['RedShift','BlueShift','NiceRedShift','NiceBlueShift']

    moneymakers = loadMoneyMakers()
    properties = [0]*len(moneymakers)

    planetmods = loadPlanetMods()
    activemods = [[] for i in xrange(len(planetmods))]

    money = 20
    moneylastclick = 0

while running:
    Clock.tick(60)
    framenum += 1
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False
    
        elif event.type == pygame.KEYDOWN:
            if event.key == pygame.K_ESCAPE:
                running = False
    
            elif event.key == pygame.K_s:
                stime = time.time()
                pygame.image.save(screen, "Screenshot_%d.png" % stime)
                print "Wrote Screenshot_%d.png" % stime
                
            elif event.key == pygame.K_SPACE:
                pass
                #Not sure why this isn't working.
#                 if currentcondition == COND_NORMAL:
#                     reset()
#                     currentcondition = COND_NORMAL
                
        elif event.type == pygame.MOUSEBUTTONDOWN:
            if currentcondition:
                continue
            #Some majick (hacks) so we don't have to do a lot of ui stuff
            if event.pos[0] < 100 and event.pos[1] < 100:
                moneyClick()
            elif 110 <= event.pos[0] <= 300:
                index = event.pos[1]
                index -= 10
                index /= 40
                if index < len(moneymakers):
                    buyShop(index)
            elif 310 <= event.pos[0] <= 400:
                index = event.pos[1]
                index -= 10
                index /= 40
                if index < len(planetmods):
                    buyMod(index)
    
    screen.fill((0, 0, 0))
    
    #Logic:
    if not currentcondition:
        moon.update()
        if len(moon.forces) == 0: #Sorry plr, but we will always change your moon
            f = random.choice(simpleforces)
            addForce(f)
        
        if moon.red >= 255 or moon.blue >= 255:
            currentcondition = COND_LOSE
        
        if moon.red + moon.blue == 0:
            currentcondition = COND_WIN
    
        for i,modlist in enumerate(activemods):
            recalc = False
            for m in modlist:
                if m.timeleft == 0:
                    modlist.remove(m)
                    recalc = True
            
            if recalc:
                planetmods[i].adjustprice(len(modlist))
    
        if framenum % 60 == 0:
            money += getIncome()
    
    
    #Render
    screen.blit(moon.mainimg,(0,0))
    screen.blit(moon.redimg,(0,0))
    screen.blit(moon.blueimg,(0,0))
    
    screen.blit(moneyfont.render("Moon Health: %d"%int(moon.red+moon.blue),True,(160,160,160)),(10,120))
    screen.blit(moneyfont.render("Money: %d"%(money),True,(160,160,160)),(10,140))
    screen.blit(moneyfont.render("Income: %d"%(getIncome()),True,(160,160,160)),(10,160))
    
    for i,mm in enumerate(moneymakers):
        screen.blit(mm.dispimg,(110,10+(i*40)))
        screen.blit(shopfont.render(mm.displayname,True,(160,160,160)),(150,10+(i*40)))
        screen.blit(shopfont.render("Price: %d"%mm.currentprice,True,(110,110,110)),(150,24+(i*40)))
        screen.blit(countfont.render(str(properties[i]),True,(40,40,40)),(120,33+(i*40)))
        
    for i,pm in enumerate(planetmods):
        screen.blit(pm.dispimg,(310,10+(i*40)))
        screen.blit(shopfont.render(pm.displayname,True,(160,160,160)),(350,10+(i*40)))
        screen.blit(shopfont.render("Price: %d"%pm.currentprice,True,(110,110,110)),(350,24+(i*40)))
        screen.blit(countfont.render(str(len(activemods[i])),True,(40,40,40)),(320,33+(i*40)))
    
    if currentcondition == COND_WIN:
        screen.blit(endfont.render("You Win!",True,(20,160,20)),(20,300))
#         screen.blit(againfont.render("Press space to play again.",True,(160,160,160)),(20,400))
        screen.blit(againfont.render("Thanks for playing!",True,(160,160,160)),(20,400))
    elif currentcondition == COND_LOSE:
        screen.blit(endfont.render("You Lose!",True,(160,20,90)),(20,300))
#         screen.blit(againfont.render("Press space to play again.",True,(160,160,160)),(20,400))
    
        screen.blit(againfont.render("Thanks for playing!",True,(160,160,160)),(20,400))
    
    pygame.display.update()

pygame.quit()